import telebot
import os
import zipfile
import random
import shutil
import json
from telebot.types import InlineKeyboardMarkup, InlineKeyboardButton
from datetime import datetime, timedelta
import threading
import time
import requests

# Main settings
BOT_TOKEN = '7513675922:AAEozlOFMK2XJX7728DaPUg1XspcfxoBqGk'
bot = telebot.TeleBot(BOT_TOKEN)
BASE_DIR = '/home/thekibo1/public_html/'
ADMIN_ID = 6703203136
ZIP_FILE = 'orc.zip'
STATS_FILE = 'bot_stats.json'
WALLET_ADDRESS = 'YOUR_TRON_WALLET_ADDRESS'  # Replace with actual Tron wallet address

# Store stats
stats_data = {
    'start_stats': [],
    'port_stats': [],
    'bot_usage': {},
    'purchases': [],
    'balances': {}
}

# Exchange rates (you might want to fetch these from an API in production)
USD_TO_TOMAN = 42000  # 1 USD = 42,000 Toman
USD_TO_TRON = 6.67    # 1 USD = 6.67 TRON (approximate)

# Pricing plans
PLANS = {
    '30_days': {'duration': 30, 'price_usd': 15},
    '20_days': {'duration': 20, 'price_usd': 10},
    '10_days': {'duration': 10, 'price_usd': 5},
    '5_days': {'duration': 5, 'price_usd': 3}
}

# Load stats from file
def load_stats():
    global stats_data
    try:
        if os.path.exists(STATS_FILE):
            with open(STATS_FILE, 'r', encoding='utf-8') as f:
                loaded_data = json.load(f)
                # Ensure all required keys exist
                for key in stats_data.keys():
                    if key not in loaded_data:
                        loaded_data[key] = stats_data[key]
                stats_data.update(loaded_data)
    except Exception as e:
        print(f"Error loading stats: {e}")
        # Initialize with default structure if loading fails
        stats_data.update({
            'start_stats': [],
            'port_stats': [],
            'bot_usage': {},
            'purchases': [],
            'balances': {}
        })

# Save stats to file
def save_stats():
    try:
        with open(STATS_FILE, 'w', encoding='utf-8') as f:
            json.dump(stats_data, f, indent=2, default=str)
    except Exception as e:
        print(f"Error saving stats: {e}")

# Load stats on startup
load_stats()

# User session management
user_sessions = {}

def generate_folder_name():
    """Generate a random 5-letter folder name"""
    return ''.join(random.choices('abcdefghijklmnopqrstuvwxyz', k=5))

def create_info_file(folder_path, user_id, bot_token, folder_name):
    """Create info.php file with requested parameters"""
    info_content = f"""<?php
$site = "https://thekibo.site";
$sudo_port = "{folder_name}";
$bot_token = "{bot_token}";
$dev_id = "Fyilk";
$dev_name = "Fyilk";
$sudo_user = ["{user_id}"];
$id_sender = "{user_id}";
//--------------------
$file = "miki-8f9a6-firebase-adminsdk-fbsvc-b13f54f00a.json";
$project_id = "miki-8f9a6";
?>"""
    
    with open(os.path.join(folder_path, 'info.php'), 'w', encoding='utf-8') as f:
        f.write(info_content)

def update_info_file(folder_path, admin_id):
    """Update info.php with admin ID after expiration"""
    info_content = f"""<?php
$site = "https://thekibo.site";
$sudo_port = "{os.path.basename(folder_path)}";
$bot_token = "";
$dev_id = "Fyilk";
$dev_name = "Fyilk";
$sudo_user = ["{admin_id}"];
$id_sender = "{admin_id}";
//--------------------
$file = "miki-8f9a6-firebase-adminsdk-fbsvc-b13f54f00a.json";
$project_id = "miki-8f9a6";
?>"""
    
    with open(os.path.join(folder_path, 'info.php'), 'w', encoding='utf-8') as f:
        f.write(info_content)

def extract_zip(zip_path, extract_to):
    """Extract zip file without modifying content"""
    try:
        with zipfile.ZipFile(zip_path, 'r') as zip_ref:
            for file in zip_ref.namelist():
                final_path = file.split('orc/')[-1] if 'orc/' in file else file
                if not final_path:
                    continue
                
                target_path = os.path.join(extract_to, final_path)
                
                if file.endswith('/'):
                    os.makedirs(target_path, exist_ok=True)
                else:
                    os.makedirs(os.path.dirname(target_path), exist_ok=True)
                    with open(target_path, 'wb') as f:
                        f.write(zip_ref.read(file))
        return True
    except Exception as e:
        print(f"Error extracting: {e}")
        return False

def create_admin_menu():
    """Create the main menu for admin"""
    markup = InlineKeyboardMarkup(row_width=2)
    markup.add(
        InlineKeyboardButton("➕ Create Port", callback_data="create_port"),
        InlineKeyboardButton("📂 View Ports", callback_data="view_ports")
    )
    markup.add(
        InlineKeyboardButton("🗑 Delete Port", callback_data="delete_port"),
        InlineKeyboardButton("📊 Start Stats", callback_data="start_stats")
    )
    markup.add(
        InlineKeyboardButton("📈 Port Stats", callback_data="port_stats"),
        InlineKeyboardButton("🔧 Bot Info", callback_data="bot_info")
    )
    markup.add(
        InlineKeyboardButton("🧹 Clear Stats", callback_data="clear_stats"),
        InlineKeyboardButton("💾 Export Stats", callback_data="export_stats")
    )
    markup.add(
        InlineKeyboardButton("💳 Charge", callback_data="charge"),
        InlineKeyboardButton("🔄 Refresh", callback_data="refresh")
    )
    return markup

def create_user_menu():
    """Create the main menu for regular users"""
    markup = InlineKeyboardMarkup(row_width=1)
    markup.add(
        InlineKeyboardButton("➕ Create Port", callback_data="create_port"),
        InlineKeyboardButton("💳 Charge", callback_data="charge")
    )
    return markup

def create_back_button():
    """Create a markup with only a Back button"""
    markup = InlineKeyboardMarkup()
    markup.add(InlineKeyboardButton("🔙 Back", callback_data="back"))
    return markup

def get_port_info(port_name):
    """Get detailed information about a specific port"""
    port_path = os.path.join(BASE_DIR, port_name)
    info = {
        'exists': os.path.exists(port_path),
        'size': 0,
        'files_count': 0,
        'created_time': None
    }
    
    if info['exists']:
        try:
            for dirpath, dirnames, filenames in os.walk(port_path):
                for filename in filenames:
                    filepath = os.path.join(dirpath, filename)
                    info['size'] += os.path.getsize(filepath)
                    info['files_count'] += 1
            
            info['created_time'] = datetime.fromtimestamp(os.path.getctime(port_path))
        except Exception as e:
            print(f"Error getting port info: {e}")
    
    return info

def format_file_size(size_bytes):
    """Format file size in human readable format"""
    if size_bytes == 0:
        return "0 B"
    size_names = ["B", "KB", "MB", "GB"]
    i = 0
    while size_bytes >= 1024 and i < len(size_names) - 1:
        size_bytes /= 1024
        i += 1
    return f"{size_bytes:.1f} {size_names[i]}"

def log_user_activity(user_id, username, action):
    """Log user activity"""
    if str(user_id) not in stats_data['bot_usage']:
        stats_data['bot_usage'][str(user_id)] = {
            'username': username,
            'actions': [],
            'first_seen': datetime.now().isoformat(),
            'last_seen': datetime.now().isoformat()
        }
    
    stats_data['bot_usage'][str(user_id)]['actions'].append({
        'action': action,
        'timestamp': datetime.now().isoformat()
    })
    stats_data['bot_usage'][str(user_id)]['last_seen'] = datetime.now().isoformat()
    save_stats()

def check_port_expiration():
    """Check and handle expired ports"""
    while True:
        current_time = datetime.now()
        for purchase in stats_data.get('purchases', []):  # Safe access with default empty list
            if purchase.get('status') == 'active':
                try:
                    expiry_time = datetime.fromisoformat(purchase['expiry_date'])
                    if current_time >= expiry_time:
                        folder_path = os.path.join(BASE_DIR, purchase['port_name'])
                        if os.path.exists(folder_path):
                            update_info_file(folder_path, ADMIN_ID)
                            purchase['status'] = 'expired'
                            bot.send_message(
                                purchase['user_id'],
                                f"ℹ️ Your port <code>{purchase['port_name']}</code> has expired.",
                                parse_mode='HTML'
                            )
                            bot.send_message(
                                ADMIN_ID,
                                f"ℹ️ Port <code>{purchase['port_name']}</code> for user {purchase['user_id']} (@{purchase['username']}) has expired and transferred to admin control.",
                                parse_mode='HTML'
                            )
                except Exception as e:
                    print(f"Error checking expiration for {purchase.get('port_name', 'unknown')}: {e}")
        save_stats()
        time.sleep(300)  # Check every 5 minutes

# Start expiration check thread
expiration_thread = threading.Thread(target=check_port_expiration, daemon=True)
expiration_thread.start()

# Auto-save stats every 5 minutes
def auto_save_stats():
    while True:
        time.sleep(300)
        save_stats()

auto_save_thread = threading.Thread(target=auto_save_stats, daemon=True)
auto_save_thread.start()

@bot.message_handler(commands=['start'])
def send_welcome(message):
    user_info = {
        'user_id': message.from_user.id,
        'username': message.from_user.username or "unknown",
        'first_name': message.from_user.first_name or "unknown",
        'timestamp': datetime.now().isoformat()
    }
    
    if not any(u['user_id'] == message.from_user.id for u in stats_data['start_stats']):
        stats_data['start_stats'].append(user_info)
        save_stats()
    
    log_user_activity(message.from_user.id, message.from_user.username or "unknown", "start_command")
    
    if message.from_user.id == ADMIN_ID:
        welcome_text = f"""<b>🔧 Port Management System</b>

👋 Welcome back, admin!

<b>📊 Quick Stats:</b>
├─ Active ports: {len([d for d in os.listdir(BASE_DIR) if os.path.isdir(os.path.join(BASE_DIR, d)) and len(d) == 5 and d.islower()])}
├─ Total users: {len(stats_data['start_stats'])}
└─ Total ports created: {len(stats_data['port_stats'])}

Select an option below:"""
        bot.send_message(message.chat.id, welcome_text, reply_markup=create_admin_menu(), parse_mode='HTML')
    else:
        balance = stats_data['balances'].get(str(message.from_user.id), 0)
        welcome_text = f"""<b>🔧 Port Management System</b>

👋 Welcome, @{message.from_user.username or 'User'}!

<b>💰 Balance:</b> ${balance:.2f}

Select an option below:"""
        bot.send_message(message.chat.id, welcome_text, reply_markup=create_user_menu(), parse_mode='HTML')

@bot.callback_query_handler(func=lambda call: True)
def handle_callbacks(call):
    log_user_activity(call.from_user.id, call.from_user.username or "unknown", f"callback_{call.data}")
    
    try:
        if call.data == "create_port":
            if call.from_user.id != ADMIN_ID and str(call.from_user.id) not in stats_data['balances']:
                bot.edit_message_text(
                    "💳 Please charge your account first!",
                    call.message.chat.id, call.message.message_id,
                    reply_markup=create_user_menu(),
                    parse_mode='HTML'
                )
                return
                
            markup = InlineKeyboardMarkup(row_width=2)
            for plan, details in PLANS.items():
                markup.add(InlineKeyboardButton(
                    f"{plan.replace('_', ' ')} | ${details['price_usd']} 🪙",
                    callback_data=f"buy_{plan}"
                ))
            markup.add(InlineKeyboardButton("🔙 Back", callback_data="back"))
            
            bot.edit_message_text(
                "<b>📦 Choose a plan:</b>",
                call.message.chat.id, call.message.message_id,
                reply_markup=markup,
                parse_mode='HTML'
            )
        
        elif call.data == "charge":
            user_sessions[call.message.chat.id] = {
                'step': 'waiting_charge_amount',
                'original_message_id': call.message.message_id
            }
            bot.edit_message_text(
                "💵 Please enter the amount in USD to charge:",
                call.message.chat.id, call.message.message_id,
                reply_markup=create_back_button(),
                parse_mode='HTML'
            )
        
        elif call.data.startswith("buy_"):
            plan = call.data[4:]
            if plan not in PLANS:
                bot.answer_callback_query(call.id, "❌ Invalid plan!")
                return
                
            user_balance = stats_data['balances'].get(str(call.from_user.id), 0)
            plan_price = PLANS[plan]['price_usd']
            
            if user_balance < plan_price and call.from_user.id != ADMIN_ID:
                bot.edit_message_text(
                    f"❌ Insufficient balance! Need ${plan_price}, you have ${user_balance:.2f}",
                    call.message.chat.id, call.message.message_id,
                    reply_markup=create_user_menu(),
                    parse_mode='HTML'
                )
                return
                
            user_sessions[call.message.chat.id] = {
                'step': 'waiting_user_id',
                'original_message_id': call.message.message_id,
                'plan': plan
            }
            bot.edit_message_text(
                "🔢 Please enter your numeric ID:",
                call.message.chat.id, call.message.message_id,
                reply_markup=create_back_button(),
                parse_mode='HTML'
            )
        
        elif call.data == "view_ports" and call.from_user.id == ADMIN_ID:
            ports = [d for d in os.listdir(BASE_DIR)
                    if os.path.isdir(os.path.join(BASE_DIR, d))
                    and len(d) == 5 and d.islower()]
            
            if ports:
                ports_text = f"<b>📂 Active Ports ({len(ports)}):</b>\n\n"
                for i, port in enumerate(ports, 1):
                    port_info = get_port_info(port)
                    ports_text += f"<code>{i}.</code> <b>{port}</b>\n"
                    ports_text += f"  ├─ 🔗 <a href='https://thekibo.site/{port}'>Link</a>\n"
                    ports_text += f"  ├─ 📊 {format_file_size(port_info['size'])}\n"
                    ports_text += f"  └─ 📁 {port_info['files_count']} files\n\n"
                
                bot.edit_message_text(
                    ports_text,
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
            else:
                bot.edit_message_text(
                    "ℹ️ No ports found",
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
        
        elif call.data == "delete_port" and call.from_user.id == ADMIN_ID:
            ports = [d for d in os.listdir(BASE_DIR)
                    if os.path.isdir(os.path.join(BASE_DIR, d))
                    and len(d) == 5 and d.islower()]
            
            if ports:
                markup = InlineKeyboardMarkup(row_width=2)
                for i in range(0, len(ports), 2):
                    row = []
                    for j in range(2):
                        if i + j < len(ports):
                            port = ports[i + j]
                            row.append(InlineKeyboardButton(f"🗑 {port}", callback_data=f"delete_{port}"))
                    markup.add(*row)
                markup.add(InlineKeyboardButton("🔙 Back", callback_data="back"))
                
                bot.edit_message_text(
                    "<b>🗑 Select port to delete:</b>",
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=markup,
                    parse_mode='HTML'
                )
            else:
                bot.edit_message_text(
                    "ℹ️ No ports available to delete",
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
        
        elif call.data == "start_stats" and call.from_user.id == ADMIN_ID:
            if stats_data['start_stats']:
                stats_message = f"<b>📊 Start Command Stats (total: {len(stats_data['start_stats'])})</b>\n\n"
                for i, user in enumerate(stats_data['start_stats'][-10:], 1):
                    stats_message += f"<code>{i}.</code> <b>ID:</b> {user['user_id']}\n"
                    stats_message += f"  ├─ <b>User:</b> @{user['username']}\n"
                    stats_message += f"  ├─ <b>Name:</b> {user['first_name']}\n"
                    stats_message += f"  └─ <b>Date:</b> {user['timestamp'][:10]}\n\n"
                
                if len(stats_data['start_stats']) > 10:
                    stats_message += f"... and {len(stats_data['start_stats']) - 10} more users"
                
                bot.edit_message_text(
                    stats_message,
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
            else:
                bot.edit_message_text(
                    "ℹ️ No start stats available",
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
        
        elif call.data == "port_stats" and call.from_user.id == ADMIN_ID:
            if stats_data['port_stats']:
                stats_message = f"<b>📈 Port Creation Stats (total: {len(stats_data['port_stats'])})</b>\n\n"
                for i, port in enumerate(stats_data['port_stats'][-10:], 1):
                    stats_message += f"<code>{i}.</code> <b>Port:</b> {port['port_name']}\n"
                    stats_message += f"  ├─ <b>User ID:</b> {port['user_id']}\n"
                    stats_message += f"  ├─ <b>Username:</b> @{port['username']}\n"
                    stats_message += f"  ├─ <b>Token:</b> {port['bot_token'][:15]}...\n"
                    stats_message += f"  └─ <b>Date:</b> {port['timestamp'][:10]}\n\n"
                
                if len(stats_data['port_stats']) > 10:
                    stats_message += f"... and {len(stats_data['port_stats']) - 10} more ports"
                
                bot.edit_message_text(
                    stats_message,
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
            else:
                bot.edit_message_text(
                    "ℹ️ No port stats available",
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
        
        elif call.data == "bot_info" and call.from_user.id == ADMIN_ID:
            uptime = datetime.now() - datetime.fromtimestamp(os.path.getctime(__file__))
            info_text = f"""<b>🤖 Bot Information</b>

<b>📊 Statistics:</b>
├─ Active ports: {len([d for d in os.listdir(BASE_DIR) if os.path.isdir(os.path.join(BASE_DIR, d)) and len(d) == 5 and d.islower()])}
├─ Total users: {len(stats_data['start_stats'])}
├─ Total ports created: {len(stats_data['port_stats'])}
├─ Active sessions: {len(user_sessions)}
└─ Uptime: {str(uptime).split('.')[0]}

<b>🔧 System:</b>
├─ Base directory: {BASE_DIR}
├─ Zip file: {ZIP_FILE}
└─ Admin ID: {ADMIN_ID}

<b>🌐 Domain:</b> <a href='https://thekibo.site/'>https://thekibo.site/</a>"""
            
            bot.edit_message_text(
                info_text,
                call.message.chat.id,
                call.message.message_id,
                reply_markup=create_back_button(),
                parse_mode='HTML'
            )
        
        elif call.data == "clear_stats" and call.from_user.id == ADMIN_ID:
            markup = InlineKeyboardMarkup()
            markup.add(
                InlineKeyboardButton("🗑 Clear Start Stats", callback_data="clear_start"),
                InlineKeyboardButton("🗑 Clear Port Stats", callback_data="clear_port")
            )
            markup.add(InlineKeyboardButton("🗑 Clear All Stats", callback_data="clear_all"))
            markup.add(InlineKeyboardButton("🔙 Back", callback_data="back"))
            
            bot.edit_message_text(
                "<b>🧹 Clear Statistics</b><br><br>What would you like to clear?",
                call.message.chat.id,
                call.message.message_id,
                reply_markup=markup,
                parse_mode='HTML'
            )
        
        elif call.data == "export_stats" and call.from_user.id == ADMIN_ID:
            try:
                export_data = {
                    'export_date': datetime.now().isoformat(),
                    'total_users': len(stats_data['start_stats']),
                    'total_ports': len(stats_data['port_stats']),
                    'active_ports': len([d for d in os.listdir(BASE_DIR) if os.path.isdir(os.path.join(BASE_DIR, d)) and len(d) == 5 and d.islower()]),
                    'stats': stats_data
                }
                
                with open(f'export_{datetime.now().strftime("%Y%m%d_%H%M%S")}.json', 'w', encoding='utf-8') as f:
                    json.dump(export_data, f, indent=2, default=str)
                
                bot.edit_message_text(
                    "<b>✅ Stats exported successfully!</b><br><br>The export file has been saved to the server.",
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
            except Exception as e:
                bot.edit_message_text(
                    f"<b>❌ Export failed:</b> {str(e)}",
                    call.message.chat.id,
                    call.message.message_id,
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
        
        elif call.data == "refresh":
            bot.edit_message_text(
                "🔄 <b>Refreshing...</b>",
                call.message.chat.id,
                call.message.message_id,
                parse_mode='HTML'
            )
            time.sleep(1)
            
            balance = stats_data['balances'].get(str(call.from_user.id), 0)
            welcome_text = f"""<b>🔧 Port Management System</b>

👋 Welcome back, {'admin' if call.from_user.id == ADMIN_ID else '@' + (call.from_user.username or 'User')}!

<b>📊 Quick Stats:</b>
├─ Active ports: {len([d for d in os.listdir(BASE_DIR) if os.path.isdir(os.path.join(BASE_DIR, d)) and len(d) == 5 and d.islower()])}
├─ Total users: {len(stats_data['start_stats'])}
└─ Total ports created: {len(stats_data['port_stats'])}

{'<b>💰 Balance:</b> $' + f'{balance:.2f}' if call.from_user.id != ADMIN_ID else ''}

Select an option below:"""
            
            bot.edit_message_text(
                welcome_text,
                call.message.chat.id,
                call.message.message_id,
                reply_markup=create_admin_menu() if call.from_user.id == ADMIN_ID else create_user_menu(),
                parse_mode='HTML'
            )
        
        elif call.data.startswith("delete_") and call.from_user.id == ADMIN_ID:
            port = call.data[7:]
            port_path = os.path.join(BASE_DIR, port)
            
            if os.path.exists(port_path):
                shutil.rmtree(port_path)
                bot.answer_callback_query(call.id, f"✅ Port {port} deleted")
                log_user_activity(call.from_user.id, call.from_user.username or "unknown", f"delete_port_{port}")
            else:
                bot.answer_callback_query(call.id, "⚠️ Port not found")
            
            welcome_text = f"""<b>🔧 Port Management System</b>

👋 Welcome back, admin!

<b>📊 Quick Stats:</b>
├─ Active ports: {len([d for d in os.listdir(BASE_DIR) if os.path.isdir(os.path.join(BASE_DIR, d)) and len(d) == 5 and d.islower()])}
├─ Total users: {len(stats_data['start_stats'])}
└─ Total ports created: {len(stats_data['port_stats'])}

Select an option below:"""
            
            bot.edit_message_text(
                welcome_text,
                call.message.chat.id,
                call.message.message_id,
                reply_markup=create_admin_menu(),
                parse_mode='HTML'
            )
        
        elif call.data.startswith("clear_") and call.from_user.id == ADMIN_ID:
            if call.data == "clear_start":
                stats_data['start_stats'] = []
                save_stats()
                bot.answer_callback_query(call.id, "✅ Start stats cleared")
            elif call.data == "clear_port":
                stats_data['port_stats'] = []
                save_stats()
                bot.answer_callback_query(call.id, "✅ Port stats cleared")
            elif call.data == "clear_all":
                stats_data['start_stats'] = []
                stats_data['port_stats'] = []
                stats_data['bot_usage'] = {}
                stats_data['purchases'] = []
                stats_data['balances'] = {}
                save_stats()
                bot.answer_callback_query(call.id, "✅ All stats cleared")
            
            welcome_text = f"""<b>🔧 Port Management System</b>

👋 Welcome back, admin!

<b>📊 Quick Stats:</b>
├─ Active ports: {len([d for d in os.listdir(BASE_DIR) if os.path.isdir(os.path.join(BASE_DIR, d)) and len(d) == 5 and d.islower()])}
├─ Total users: {len(stats_data['start_stats'])}
└─ Total ports created: {len(stats_data['port_stats'])}

Select an option below:"""
            
            bot.edit_message_text(
                welcome_text,
                call.message.chat.id,
                call.message.message_id,
                reply_markup=create_admin_menu(),
                parse_mode='HTML'
            )
        
        elif call.data == "back":
            balance = stats_data['balances'].get(str(call.from_user.id), 0)
            welcome_text = f"""<b>🔧 Port Management System</b>

👋 Welcome back, {'admin' if call.from_user.id == ADMIN_ID else '@' + (call.from_user.username or 'User')}!

<b>📊 Quick Stats:</b>
├─ Active ports: {len([d for d in os.listdir(BASE_DIR) if os.path.isdir(os.path.join(BASE_DIR, d)) and len(d) == 5 and d.islower()])}
├─ Total users: {len(stats_data['start_stats'])}
└─ Total ports created: {len(stats_data['port_stats'])}

{'<b>💰 Balance:</b> $' + f'{balance:.2f}' if call.from_user.id != ADMIN_ID else ''}

Select an option below:"""
            
            bot.edit_message_text(
                welcome_text,
                call.message.chat.id,
                call.message.message_id,
                reply_markup=create_admin_menu() if call.from_user.id == ADMIN_ID else create_user_menu(),
                parse_mode='HTML'
            )
    
    except Exception as e:
        print(f"Error in callback handler: {e}")
        bot.answer_callback_query(call.id, f"❌ Error: {str(e)}")

@bot.message_handler(func=lambda message: True)
def handle_messages(message):
    chat_id = message.chat.id
    
    if chat_id in user_sessions:
        session = user_sessions[chat_id]
        
        if session['step'] == 'waiting_charge_amount':
            try:
                amount_usd = float(message.text.strip())
                if amount_usd <= 0:
                    raise ValueError("Amount must be positive")
                
                amount_toman = amount_usd * USD_TO_TOMAN
                amount_tron = amount_usd * USD_TO_TRON
                
                charge_text = f"""<b>💳 Charge Request</b>

<b>📌 Details:</b>
├─ USD: ${amount_usd:.2f}
├─ Toman: {amount_toman:,.0f} IRR
├─ TRON: {amount_tron:.2f} TRX
└─ Wallet Address: <code>{WALLET_ADDRESS}</code>

Please send the payment to the provided wallet address and reply with the transaction ID."""
                
                user_sessions[chat_id]['charge_amount'] = amount_usd
                user_sessions[chat_id]['step'] = 'waiting_transaction_id'
                
                bot.edit_message_text(
                    charge_text,
                    chat_id,
                    session['original_message_id'],
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
                
                try:
                    bot.delete_message(chat_id, message.message_id)
                except:
                    pass
                
            except ValueError:
                bot.send_message(
                    chat_id,
                    "❌ Please enter a valid amount!",
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
                try:
                    bot.delete_message(chat_id, message.message_id)
                except:
                    pass
        
        elif session['step'] == 'waiting_transaction_id':
            transaction_id = message.text.strip()
            # In a real implementation, verify the transaction here
            # For now, assume the transaction is valid
            
            amount_usd = session['charge_amount']
            user_id = str(message.from_user.id)
            
            stats_data['balances'][user_id] = stats_data['balances'].get(user_id, 0) + amount_usd
            save_stats()
            
            bot.edit_message_text(
                f"<b>✅ Account Charged Successfully!</b><br><br>New Balance: ${stats_data['balances'][user_id]:.2f}",
                chat_id,
                session['original_message_id'],
                reply_markup=create_user_menu() if message.from_user.id != ADMIN_ID else create_admin_menu(),
                parse_mode='HTML'
            )
            
            bot.send_message(
                ADMIN_ID,
                f"💳 New charge by @{message.from_user.username or 'User'} (ID: {user_id})<br>"
                f"Amount: ${amount_usd:.2f}<br>"
                f"Transaction ID: {transaction_id}",
                parse_mode='HTML'
            )
            
            del user_sessions[chat_id]
            try:
                bot.delete_message(chat_id, message.message_id)
            except:
                pass
        
        elif session['step'] == 'waiting_user_id':
            try:
                user_id = int(message.text.strip())
                session['user_id'] = user_id
                session['step'] = 'waiting_bot_token'
                
                sent_message = bot.send_message(
                    chat_id,
                    "🤖 Please enter bot token:",
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
                session['current_message_id'] = sent_message.message_id
                
                try:
                    bot.delete_message(chat_id, message.message_id)
                except:
                    pass
                    
            except ValueError:
                bot.send_message(
                    chat_id,
                    "❌ ID must be numeric!",
                    reply_markup=create_back_button(),
                    parse_mode='HTML'
                )
                try:
                    bot.delete_message(chat_id, message.message_id)
                except:
                    pass
        
        elif session['step'] == 'waiting_bot_token':
            try:
                bot_token = message.text.strip()
                user_id = session['user_id']
                plan = session.get('plan')
                
                folder_name = generate_folder_name()
                folder_path = os.path.join(BASE_DIR, folder_name)
                
                os.makedirs(folder_path, exist_ok=True)
                
                if extract_zip(ZIP_FILE, folder_path):
                    create_info_file(folder_path, user_id, bot_token, folder_name)
                    
                    port_info = {
                        'port_name': folder_name,
                        'bot_token': bot_token,
                        'user_id': user_id,
                        'username': message.from_user.username or "unknown",
                        'first_name': message.from_user.first_name or "unknown",
                        'timestamp': datetime.now().isoformat()
                    }
                    stats_data['port_stats'].append(port_info)
                    
                    if message.from_user.id != ADMIN_ID:
                        plan_price = PLANS[plan]['price_usd']
                        duration = PLANS[plan]['duration']
                        stats_data['balances'][str(message.from_user.id)] -= plan_price
                        stats_data['purchases'].append({
                            'user_id': message.from_user.id,
                            'username': message.from_user.username or "unknown",
                            'port_name': folder_name,
                            'plan': plan,
                            'price_usd': plan_price,
                            'purchase_date': datetime.now().isoformat(),
                            'expiry_date': (datetime.now() + timedelta(days=duration)).isoformat(),
                            'status': 'active'
                        })
                        
                        bot.send_message(
                            ADMIN_ID,
                            f"🛒 New purchase by @{message.from_user.username or 'User'} (ID: {user_id})<br>"
                            f"Plan: {plan.replace('_', ' ')}<br>"
                            f"Price: ${plan_price}<br>"
                            f"Port: {folder_name}<br>"
                            f"Duration: {duration} days",
                            parse_mode='HTML'
                        )
                    
                    save_stats()
                    
                    log_user_activity(message.from_user.id, message.from_user.username or "unknown", f"create_port_{folder_name}")
                    
                    success_text = f"""<b>✅ Port created successfully!</b>

<b>📌 Details:</b>
├─ <b>Port name:</b> <code>{folder_name}</code>
├─ <b>Address:</b> <a href='https://thekibo.site/{folder_name}'>Click here</a>
├─ <b>Your ID:</b> <code>{user_id}</code>
├─ <b>Bot token:</b> <code>{bot_token[:20]}...</code>
└─ <b>Created:</b> {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}

{'└─ <b>Expires:</b> ' + (datetime.now() + timedelta(days=PLANS[plan]['duration'])).strftime('%Y-%m-%d %H:%M:%S') if plan else ''}

<b>🎉 Your port is ready to use!</b>"""
                    
                    if 'current_message_id' in session:
                        bot.edit_message_text(
                            success_text,
                            chat_id,
                            session['current_message_id'],
                            reply_markup=create_user_menu() if message.from_user.id != ADMIN_ID else create_admin_menu(),
                            parse_mode='HTML'
                        )
                    else:
                        bot.send_message(
                            chat_id,
                            success_text,
                            reply_markup=create_user_menu() if message.from_user.id != ADMIN_ID else create_admin_menu(),
                            parse_mode='HTML'
                        )
                else:
                    raise Exception("Error extracting files")
                    
            except Exception as e:
                error_text = f"<b>❌ Error:</b> {str(e)}"
                if 'current_message_id' in session:
                    bot.edit_message_text(
                        error_text,
                        chat_id,
                        session['current_message_id'],
                        reply_markup=create_user_menu() if message.from_user.id != ADMIN_ID else create_admin_menu(),
                        parse_mode='HTML'
                    )
                else:
                    bot.send_message(
                        chat_id,
                        error_text,
                        reply_markup=create_user_menu() if message.from_user.id != ADMIN_ID else create_admin_menu(),
                        parse_mode='HTML'
                    )
            finally:
                if chat_id in user_sessions:
                    del user_sessions[chat_id]
                
                try:
                    bot.delete_message(chat_id, message.message_id)
                except:
                    pass
    else:
        bot.reply_to(message, "🤖 Use /start to begin", parse_mode='HTML')

if __name__ == '__main__':
    print("🤖 Bot started...")
    try:
        bot.infinity_polling(timeout=10, long_polling_timeout=5)
    except Exception as e:
        print(f"Error: {e}")
        time.sleep(5)
        bot.infinity_polling(timeout=10, long_polling_timeout=5)